"""Define a client to interact with Pollen.com."""
from urllib.parse import urlparse

from aiohttp import ClientSession, client_exceptions

from .allergens import Allergens
from .asthma import Asthma
from .disease import Disease
from .errors import RequestError

API_USER_AGENT = 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_13_4) ' \
    + 'AppleWebKit/537.36 (KHTML, like Gecko) ' \
    + 'Chrome/65.0.3325.181 Safari/537.36'


class Client:  # pylint: disable=too-few-public-methods
    """Define the client."""

    def __init__(self, zip_code: str, websession: ClientSession) -> None:
        """Initialize."""
        self._websession = websession
        self.zip_code = zip_code

        self.allergens = Allergens(self._request)
        self.asthma = Asthma(self._request)
        self.disease = Disease(self._request)

    async def _request(
            self,
            method: str,
            url: str,
            *,
            headers: dict = None,
            params: dict = None,
            json: dict = None) -> dict:
        """Make a request against AirVisual."""
        full_url = '{0}/{1}'.format(url, self.zip_code)
        pieces = urlparse(url)

        if not headers:
            headers = {}
        headers.update({
            'Content-Type': 'application/json',
            'Referer': '{0}://{1}'.format(pieces.scheme, pieces.netloc),
            'User-Agent': API_USER_AGENT
        })

        async with self._websession.request(method, full_url, headers=headers,
                                            params=params, json=json) as resp:
            try:
                resp.raise_for_status()
                data = await resp.json(content_type=None)
                return data
            except client_exceptions.ClientError as err:
                raise RequestError(
                    'Error requesting data from {0}: {1}'.format(url, err))



"""Define errors."""


class PollenComError(Exception):
    """Define a base package error."""


class InvalidZipError(PollenComError):
    """Define an error when a ZIP returns no valid data."""


class RequestError(PollenComError):
    """Define a generic request error."""



"""Define useful decorators."""
from typing import Callable

from .errors import InvalidZipError


def raise_on_invalid_zip(func: Callable) -> Callable:
    """Raise an exception when there's no data (via a bad ZIP code)."""
    async def decorator(*args: list, **kwargs: dict) -> dict:
        """Decorate."""
        data = await func(*args, **kwargs)
        if not data['Location']['periods']:
            raise InvalidZipError('No data returned for ZIP code')
        return data

    return decorator



"""Define an object to work with "Allergens" endpoints."""
from typing import Awaitable, Callable

from .decorators import raise_on_invalid_zip
from .errors import InvalidZipError, RequestError


class Allergens:
    """Define the "Allergens" object."""

    def __init__(self, request: Callable[..., Awaitable[dict]]) -> None:
        """Initialize."""
        self._request = request

    @raise_on_invalid_zip
    async def current(self) -> dict:
        """Get current allergy conditions."""
        return await self._request(
            'get', 'https://www.pollen.com/api/forecast/current/pollen')

    @raise_on_invalid_zip
    async def extended(self) -> dict:
        """Get extended allergen info."""
        return await self._request(
            'get', 'https://www.pollen.com/api/forecast/extended/pollen')

    @raise_on_invalid_zip
    async def historic(self) -> dict:
        """Get historic allergen info."""
        return await self._request(
            'get', 'https://www.pollen.com/api/forecast/historic/pollen')

    async def outlook(self) -> dict:
        """Get allergen outlook."""
        try:
            return await self._request(
                'get', 'https://www.pollen.com/api/forecast/outlook')
        except RequestError as err:
            if '404' in str(err):
                raise InvalidZipError('No data returned for ZIP code')
            else:
                raise RequestError(err)
